library(openxlsx)
library(tidyverse)
library(qgraph)
library(bootnet)
library(EstimateGroupNetwork)
library(NetworkComparisonTest)
library(networktools)
library(bnlearn)
library(Rgraphviz)

#Part 1: CAPS Network (Patient and Control)

#Paitent CAPS
#load data
setwd("/Users/danielcoppersmith/Documents/Harvard_PhD/Projects/InProgress/PTSDNetworkAnalysis/PTSD_IDF_Network/Data")
PTSD.dat<-read.xlsx("PTSD_IDF_L.xlsx", sheet = 1, colNames = TRUE)
#check data
head(PTSD.dat)
#select group: patient
PTSD.dat<- subset(PTSD.dat, dx == 1)
head(PTSD.dat)
#make a dataframe with just the variables for the network#
PTSD.dat<-PTSD.dat[,2:18]
#remove missing data#
PTSD.dat<-drop_na(PTSD.dat)
head(PTSD.dat)

#Control CAPS
#load data
PTSDHV.dat<-read.xlsx("PTSD_IDF_L.xlsx", sheet = 1, colNames = TRUE)
#check data
head(PTSDHV.dat)
#select group: patient
PTSDHV.dat<- subset(PTSDHV.dat, dx == 0)
head(PTSDHV.dat)
#make a dataframe with just the variables for the network#
PTSDHV.dat<-PTSDHV.dat[,2:18]
#check that the newdata frame has just the variables for the network (e.g. no subjectIDs)
head(PTSDHV.dat)
#remove missing data#
PTSDHV.dat<-drop_na(PTSDHV.dat)

#create labels for network
net_namesCAPS<-c("INTRU", "DREAM", "FLASH", "UPSET", "PHY", "AVTHT","AVSIT","AMNES","DISINT","DTACH","NUMB","FUTRE","SLEEP","ANGER","CONC","HYPER","STRTL")

#Estimate the joint network
EGN1 <- EstimateGroupNetwork(list(PTSD.dat, PTSDHV.dat), method="crossvalidation", 
                            strategy="sequential", simplifyOutput = FALSE, seed=1337, ncores=8, covfun = cor_auto)

#plot networks
g1 <- qgraph(EGN1$network[[1]], layout = "circle", title = "A.", theme="colorblind",
             border.width=1, vsize=10, border.color='#555555', label.color="#555555", color="#EEEEEE") 
g2 <- qgraph(EGN1$network[[2]], layout = "circle", title = "B.", theme="colorblind",
             border.width=1, vsize=10, border.color='#555555', label.color="#555555", color="#EEEEEE") 

#centrality values patient CAPS Network
pdf("Figure2.pdf", width=10, height=10)
par(mfrow=c(1,1))
centralityPlot(g1, include = "ExpectedInfluence", orderBy = "ExpectedInfluence")
dev.off()   

#calculate edge weight accuracy paitent CAPS Network
Network <- estimateNetwork(PTSD.dat, default = "EBICglasso")
boot1 <- bootnet(Network, nBoots = 1000, nCores = 8)
#plot edge weight accuracy
accuracy1 <- plot(boot1, labels = FALSE, order = "sample")
plot(Acc)
#plot centrality (expected influence) stability
boot2 <- bootnet(Network, statistics = "ExpectedInfluence", 
                 nBoots = 1000, nCores = 8, type = "case")

stability1<-plot(boot2, statistics = "ExpectedInfluence") + 
  theme(legend.position = "none")

#calculate corstability
corStability(boot2)

#generate figure 3
pdf("Fig3.pdf")
par(mfrow=c(2,2))
plot(accuracy1)
plot(stability1)
dev.off()   

#plot sig diff edges
plot(boot1, "edge", plot = "difference", onlyNonZero = TRUE, order = "sample")

#plot sig diff nodes
plot(boot1, "Strength")

##centrality values control CAPS Network
centralityPlot(g2, include = "ExpectedInfluence", orderBy = "ExpectedInfluence")

#calculate edge weight accuracy control CAPS Network
NetworkHV <- estimateNetwork(PTSDHV.dat, default = "EBICglasso")

boot3 <- bootnet(NetworkHV, nBoots = 1000, nCores = 8)
#plot edge weight accuracy
AccHV <- plot(boot3, labels = FALSE, order = "sample")
plot(AccHV)
#plot centrality (expected influence) stability
boot4 <- bootnet(NetworkHV, statistics = "ExpectedInfluence", 
                 nBoots = 1000, nCores = 8, type = "case")
stabilityHV<-plot(boot4, statistics = "ExpectedInfluence") + 
  theme(legend.position = "none")

#calculate corstability control CAPS Network
corStability(boot4)

#plot sig diff edges control CAPS Network
plot(boot3, "edge", plot = "difference", onlyNonZero = TRUE, order = "sample")

#plot sig diff nodes control CAPS Network
plot(boot3, "ExpectedInfluence")

#network comparison test
set.seed(123)
compare <- NCT(sample_n(PTSD.dat, 306),PTSDHV.dat, it=5000, binary.data=FALSE, test.edges=TRUE, edges='all', progressbar=TRUE)

#NCT maximum difference in edge weights
compare$nwinv.pval

# quantification of differences: count significantly different edges (total number 120)
sum(compare$einv.pvals$"p-value" < 0.05)
#11 significantly different edges

# NCT global strength
compare$glstrinv.pval 

##Part 2: IDF PATIENT CAPS + MADRS Network
#load data
PTSDMDD.dat<-read.xlsx("IDF_DAG.xlsx", sheet = 1, colNames = TRUE)
#check data
head(PTSDMDD.dat)
#select group: patient
PTSDMDD.dat<- subset(PTSDMDD.dat, dx == 1)
head(PTSDMDD.dat)
#make a dataframe with just the variables for the network#
PTSDMDD.dat<-PTSDMDD.dat[,3:29]
#check that the newdata frame has just the variables for the network (e.g. no subjectIDs)
head(PTSDMDD.dat)
#remove missing data#
PTSDMDD.dat<-drop_na(PTSDMDD.dat)
head(PTSDMDD.dat)
#create labels for network#
net_namesCAPSMDD<-c("INTRU","DREAM", "FLASH", "UPSET", "PHY", "AVTHT","AVSIT","AMNES","DISINT","DTACH","NUMB","FUTRE","SLEEP","ANGER","CONC","HYPER","STRTL", "APPSAD", "REPSAD", "TENSION", "RESLEEP", "REAPP", "CONCDIFF", "LASS", "FEEL", "PESS", "SUI")
#estimate and plot network
NetworkMDD <- estimateNetwork(PTSDMDD.dat, default = "EBICglasso")
plot(NetworkMDD)
Groups = c(rep("PTSD",17),rep("MDD",10))
#Names is the labels that will appear next to the graph
Colors = c("steelblue1","firebrick1")
#Plot Network
PatientNet_MDD <- plot(NetworkMDD, layout = "spring", sampleSize = nrow(PTSD.dat), labels = net_namesCAPSMDD, title = "C.",
                       groups = Groups, colors = Colors)
MDSnet(PatientNet_MDD, repluse=T, repulsion = 1)

#Create figure 1
pdf("Figure1.pdf", width=10, height=10)
par(mfrow=c(2,2))
g1 <- qgraph(EGN1$network[[1]], layout = "circle", title = "A.", theme="colorblind",
             border.width=1, vsize=10, border.color='#555555', label.color="#555555", color="#EEEEEE") 
g2 <- qgraph(EGN1$network[[2]], layout = "circle", title = "B.", theme="colorblind",
             border.width=1, vsize=10, border.color='#555555', label.color="#555555", color="#EEEEEE") 
MDSnet(PatientNet_MDD, repluse=T, repulsion = 1)
dev.off() 

#plot centrality
centralityPlot(NetworkMDD, include = "ExpectedInfluence", labels = net_namesCAPSMDD, orderBy = "ExpectedInfluence")

#calculate bridge metrics
PTSDMDD.dat.cor.matrix<-cor_auto(PTSDMDD.dat)
PTSDMDD.partial.corr.network<-qgraph(PTSDMDD.dat.cor.matrix, graph="EBICglasso", labels = net_namesCAPSMDD, layout = "spring", sampleSize=nrow(PTSDMDD.dat))
b <- bridge(PTSDMDD.partial.corr.network,communities = c(rep('1',17), rep('2',10)), useCommunities = "all", directed = NULL, nodes = NULL)
bridge_expectedinfluence<-plot(b, order="value", include = "Bridge Expected Influence (1-step)", zscore=TRUE)

#calculate edge weight accuracy
boot5 <- bootnet(NetworkMDD, nBoots =1000, nCores = 8)
#plot edge weight accuracy
plot(boot5, labels = FALSE, order = "sample")
#plot centrality stability
boot6 <- bootnet(NetworkMDD, statistics = "ExpectedInfluence", type = "case")
plot(boot6, statistics = "ExpectedInfluence")
#calculate corstability - a metric of centrality stability
corStability(boot6)
#plot sig diff edges
plot(boot5, "edge", plot = "difference",
     onlyNonZero = TRUE, order = "sample")
#plot sig diff nodes
plot(boot5, "Strength")


#Part 4: DAG with CAPS + MADRS + FUNCTION

#Part 1 IDF Patient CAPS Network
#load data
PTSDDAG.dat<-read.xlsx("IDF_DAG.xlsx", sheet = 1, colNames = TRUE)
#check data
head(PTSDDAG.dat)
#select group: patient
PTSDDAG.dat<- subset(PTSDDAG.dat, dx == 1)
head(PTSDDAG.dat)
#make a dataframe with just the variables for the network#
PTSDDAG.dat<-PTSDDAG.dat[,3:30]
#remove missing data#
PTSDDAG.dat<-drop_na(PTSDDAG.dat)
head(PTSDDAG.dat)

## Hill Climbing Algorithm-- Bayesian network learned via Score-based methods & network averaging (bootstrap based)
set.seed(123)
bootnet <- bnlearn::boot.strength(PTSDDAG.dat, R = 1000, algorithm = "hc", algorithm.args = list(restart = 5, perturb = 10), debug = TRUE, cpdag = TRUE)
head(bootnet)

## net1: use the optimal cutpoint according to Scurati & Nagarajan (2013) 
avgnet1 <- bnlearn::averaged.network(bootnet)
avgnet1      
bnlearn::score(avgnet1, data = PTSDDAG.dat)
thresh <- avgnet1$learning$args[[1]]
thresh                                  ## optimal significance threshold
astr1 <- bnlearn::arc.strength(avgnet1, PTSDDAG.dat, "bic-g")   ## compute edge strengths
astr1

##plot DAG
suppressWarnings(bnlearn::strength.plot(avgnet1, astr1, shape = "ellipse", threshold = thresh))

## Save DAG as a PDF
pdf("Figure4.pdf", width = 12)
suppressWarnings(bnlearn::strength.plot(avgnet1, astr1, shape = "ellipse", threshold = thresh))
dev.off()

#Extract data for reproducibility
#joint networks
save(g1,g2, file="JointEstimation.RData")
#PTSD patients network
ptsdnet<-cor_auto(PTSD.dat)
save(ptsdnet, file="PaitentCAPS.RData")
#PTSD control network
ptsdnet<-cor_auto(PTSD.dat)
save(ptsdnet, file="PaitentCAPS.RData")
#PTSD control network
ptsdhvnet<-cor_auto(PTSDHV.dat)
save(ptsdhvnet, file="ControlCAPS.RData")
#PTSD-MDD network
ptsdmddnet<-cor_auto(PTSDMDD.dat)
save(ptsdmddnet, file="CAPSMADRS.RData")
#DAG network
save(bootnet, file="DAG.RData")
